@@ -521,6 +521,10 @@ static char ** merge_unresolved(char ** oldstuff, char ** newstuff)
     while(newstuff && newstuff[newlen]) newlen++;
     
     result = (char **)realloc(oldstuff, sizeof(char *) * (oldlen + newlen + 1));
+    if (result == NULL) {
+        fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+        return NULL;
+    }
     
     for(i = oldlen, j = 0; i < (oldlen + newlen); i++, j++)
 	*(result + i) = *(newstuff + j);
@@ -545,6 +549,10 @@ char ** add_unresolved_dep(pkg_t * pkg, char ** the_lost, int ref_ndx)
 
     count++;  /* need one to hold the null */
     resized = (char **)realloc(the_lost, sizeof(char *) * (count + 1));
+    if (resized == NULL) {
+       fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+       return NULL;
+    }
     resized[count - 1] = strdup(depend_str);
     resized[count] = NULL;
     
@@ -577,7 +585,7 @@ void printDepends(pkg_t * pkg)
     }
 }
 
-void buildProvides(hash_table_t * hash, abstract_pkg_t * ab_pkg, pkg_t * pkg)
+int buildProvides(hash_table_t * hash, abstract_pkg_t * ab_pkg, pkg_t * pkg)
 {
     register int i, j;
 
@@ -585,9 +593,13 @@ void buildProvides(hash_table_t * hash, abstract_pkg_t * ab_pkg, pkg_t * pkg)
     abstract_pkg_vec_insert(ab_pkg->provided_by, ab_pkg);
 
     if (!pkg->provides_count)
-      return;
+      return 0;
 
     pkg->provides = (abstract_pkg_t **)malloc(sizeof(abstract_pkg_t *) * (pkg->provides_count + 1));
+    if (pkg->provides == NULL) {
+       fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+       return -1 ;
+    }
     pkg->provides[0] = ab_pkg;
 
     // if (strcmp(ab_pkg->name, pkg->name))
@@ -601,19 +613,24 @@ void buildProvides(hash_table_t * hash, abstract_pkg_t * ab_pkg, pkg_t * pkg)
 	j = 0;
 	abstract_pkg_vec_insert(provided_abpkg->provided_by, ab_pkg);
     }
+    return 0;
 }
 
 /* Abhaya: added conflicts support */
-void buildConflicts(hash_table_t * hash, abstract_pkg_t * ab_pkg, pkg_t * pkg)
+int buildConflicts(hash_table_t * hash, abstract_pkg_t * ab_pkg, pkg_t * pkg)
 {
     register int i;
     compound_depend_t * conflicts;
 
     if (!pkg->conflicts_count)
-	return;
+	return 0;
 
     conflicts = pkg->conflicts = malloc(sizeof(compound_depend_t) *
 					pkg->conflicts_count);
+    if (conflicts == NULL) {
+       fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+       return  -1;
+    }
     for (i = 0; i < pkg->conflicts_count; i++) {
 	 conflicts->type = CONFLICTS;
 	 parseDepends(conflicts, hash,
@@ -627,16 +644,21 @@ void buildConflicts(hash_table_t * hash, abstract_pkg_t * ab_pkg, pkg_t * pkg)
 #endif
 	 conflicts++;
     }
+    return 0;
 }
 
-void buildReplaces(hash_table_t * hash, abstract_pkg_t * ab_pkg, pkg_t * pkg)
+int buildReplaces(hash_table_t * hash, abstract_pkg_t * ab_pkg, pkg_t * pkg)
 {
      register int i, j;
 
      if (!pkg->replaces_count)
-	  return;
+	  return 0;
 
      pkg->replaces = (abstract_pkg_t **)malloc(sizeof(abstract_pkg_t *) * pkg->replaces_count);
+     if (pkg->replaces == NULL) {
+        fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+        return  -1;
+     }
 
      // if (strcmp(ab_pkg->name, pkg->name))
      //     fprintf(stderr, __FUNCTION__ ": ab_pkg=%s pkg=%s\n", ab_pkg->name, pkg->name);
@@ -649,27 +671,36 @@ void buildReplaces(hash_table_t * hash, abstract_pkg_t * ab_pkg, pkg_t * pkg)
 	  j = 0;
 	  if (!old_abpkg->replaced_by)
 	       old_abpkg->replaced_by = abstract_pkg_vec_alloc();
+               if ( old_abpkg->replaced_by == NULL ){
+                  return -1;
+               }
 	  /* if a package pkg both replaces and conflicts old_abpkg,
 	   * then add it to the replaced_by vector so that old_abpkg
 	   * will be upgraded to ab_pkg automatically */
 	  if (pkg_conflicts_abstract(pkg, old_abpkg))
 	       abstract_pkg_vec_insert(old_abpkg->replaced_by, ab_pkg);
      }
+     return 0;
 }
 
-void buildDepends(hash_table_t * hash, pkg_t * pkg)
+int buildDepends(hash_table_t * hash, pkg_t * pkg)
 {
      int count;
      register int i;
      compound_depend_t * depends;
 
      if(!(count = pkg->pre_depends_count + pkg->depends_count + pkg->recommends_count + pkg->suggests_count))
-	  return;
+	  return 0;
 
      if (0 && pkg->pre_depends_count)
 	  fprintf(stderr, "pkg=%s pre_depends_count=%d depends_count=%d\n", 
 		  pkg->name, pkg->pre_depends_count, pkg->depends_count);
      depends = pkg->depends = malloc(sizeof(compound_depend_t) * count);
+     if (depends == NULL) {
+        fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+        return  -1;
+     }
+     
 
      for(i = 0; i < pkg->pre_depends_count; i++){
 	  parseDepends(depends, hash, pkg->pre_depends_str[i]);
@@ -705,6 +736,7 @@ void buildDepends(hash_table_t * hash, pkg_t * pkg)
 		       pkg->depends_str[i], depends, depends->possibility_count);
 	  depends++;
      }
+     return 0;
 }    
 
 /*
@@ -798,7 +830,10 @@ void buildDependedUponBy(pkg_t * pkg, abstract_pkg_t * ab_pkg)
 static depend_t * depend_init(void)
 {
     depend_t * d = (depend_t *)malloc(sizeof(depend_t));    
-    
+    if ( d==NULL ){
+        fprintf(stderr, "%s: out of memory\n", __FUNCTION__);
+        return NULL; 
+     }
     d->constraint = NONE;
     d->version = NULL;
     d->pkg = NULL;
@@ -837,6 +872,8 @@ static int parseDepends(compound_depend_t *compound_depend,
      src = depend_str;
      for(i = 0; i < num_of_ors + 1; i++){
 	  possibilities[i] = depend_init();
+          if (!possibilities[i])
+	       return -ENOMEM;
 	  /* gobble up just the name first */
 	  dest = buffer;
 	  while(*src &&
@@ -847,6 +884,8 @@ static int parseDepends(compound_depend_t *compound_depend,
 	       *dest++ = *src++;
 	  *dest = '\0';
 	  pkg_name = trim_alloc(buffer);
+          if (pkg_name == NULL )
+	       return -ENOMEM;
 	
 	  /* now look at possible version info */
 	
@@ -899,6 +938,8 @@ static int parseDepends(compound_depend_t *compound_depend,
 	       possibilities[i]->version = trim_alloc(buffer);
 	       /*	   	    fprintf(stderr, "let's print the depends version string:");
 				    fprintf(stderr, "version %s\n", possibilities[i]->version);*/
+               if (possibilities[i]->version == NULL )
+	            return -ENOMEM;
 
 	 
 	  }
